@@ -226,15 +226,24 @@ fn exists_object(
 
 fn drop_object(
     pg_client: &mut postgres::Client,
-    object_id: &str
+    object_id: &str,
+    force: bool,
 ) -> anyhow::Result<()> {
     println!("drop if exists {:?}", object_id);
     let object_type = project::get_object_type(object_id)?;
     let exists = exists_object(pg_client, &object_id)?;
     if exists {
         match object_type {
-            DatabaseObjectType::Table => bail!("attempting to drop a table, \
-                tables should be dropped manually or using migration scripts {:?}", object_id),
+            DatabaseObjectType::Table => {
+                if !force {
+                    bail!("attempting to drop a table, \
+                    tables should be dropped manually or using migration scripts {:?}", object_id)
+                }
+                let schema = project::get_schema(object_id)?;
+                let name = project::get_name(object_id)?;
+                let sql = format!("drop table {}.{};", schema, name);
+                pg_client.batch_execute(&sql)?;
+            },
             DatabaseObjectType::View => {
                 let schema = project::get_schema(object_id)?;
                 let name = project::get_name(object_id)?;
@@ -348,6 +357,7 @@ fn drop_object_with_deps(
     object: &DatabaseObject,
     database_project: &DatabaseProject,
     objects: &HashMap<String, DatabaseObject>,
+    force: bool,
     dropped: &mut HashSet<String>,
     visited: &mut HashSet<String>,
 ) -> anyhow::Result<()> {
@@ -361,7 +371,7 @@ fn drop_object_with_deps(
     visited.insert(object.id.clone());
 
     // first attempt to drop the target without dropping dependencies
-    let drop_result = drop_object(pg_client, &object.id);
+    let drop_result = drop_object(pg_client, &object.id, force);
     if drop_result.is_ok() {
         dropped.insert(object.id.clone());
         return Ok(());
@@ -374,6 +384,7 @@ fn drop_object_with_deps(
                 &dep,
                 &database_project,
                 &objects,
+                force,
                 dropped,
                 visited
             )?;
@@ -384,17 +395,18 @@ fn drop_object_with_deps(
                 &dep,
                 &database_project,
                 &objects,
+                force,
                 dropped,
                 visited
             )?;
 
         } else {
-            drop_object(pg_client, dep_id)
+            drop_object(pg_client, dep_id, force)
                 .context(format!("undefined dependency could not be dropped {:?} {:?}", object.id, dep_id))?;
         }
     }
 
-    drop_object(pg_client, &object.id)?;
+    drop_object(pg_client, &object.id, force)?;
     dropped.insert(object.id.clone());
     return Ok(());
 }
@@ -681,7 +693,8 @@ fn update_objects(
                 pg_client, 
                 &object, 
                 &database_project,
-                &db_objects, 
+                &db_objects,
+                false,
                 &mut dropped,
                 &mut visited
             );
@@ -829,10 +842,10 @@ pub fn migrate(database_project: DatabaseProject) -> anyhow::Result<()> {
 }
 
 
-pub fn drop(database_project: DatabaseProject) -> anyhow::Result<()> {
+pub fn drop_database(database_project: &DatabaseProject) -> anyhow::Result<()> {
 
     let mut pg_admin_client = get_admin_pg_client()
-        .context("drop error: failed to connect as admin")?;
+        .context("drop database error: failed to connect as admin")?;
 
 
     let pg_client_result = get_pg_client();
@@ -844,7 +857,7 @@ pub fn drop(database_project: DatabaseProject) -> anyhow::Result<()> {
                 let object_type = db_object.object_type()?;
                 if object_type == DatabaseObjectType::Role {
                     force_drop_role_if_exists(&mut pg_client, db_object_id)
-                        .context(format!("drop error: failed to drop role, drop it manually or remove it from pgfine_objects table {:?}", db_object_id))?;
+                        .context(format!("drop database error: failed to drop role, drop it manually or remove it from pgfine_objects table {:?}", db_object_id))?;
                 }
             }
         
@@ -852,28 +865,28 @@ pub fn drop(database_project: DatabaseProject) -> anyhow::Result<()> {
                 let object_type = p_object.object_type()?;
                 if object_type == DatabaseObjectType::Role {
                     force_drop_role_if_exists(&mut pg_client, p_object_id)
-                        .context(format!("drop error: failed to drop role, drop it manually or remove it from the project {:?}", p_object_id))?;
+                        .context(format!("drop database error: failed to drop role, drop it manually or remove it from the project {:?}", p_object_id))?;
                 }
             }
 
             pg_client.close()
-                .context("drop error: could not properly close connection before dropping database, try again?")?;
+                .context("drop database error: could not properly close connection before dropping database, try again?")?;
         },
         Err(err) => {
             let exists = exists_database(&mut pg_admin_client)?;
             if exists {
                 return Err(err)
-                    .context("drop error: database exists but could not get connection to it, check PGFINE_CONNECTION_STRING");
+                    .context("drop database error: database exists but could not get connection to it, check PGFINE_CONNECTION_STRING");
             }
         }
     }
 
-    for (path_buf, script) in database_project.drop_scripts {
+    for (path_buf, script) in database_project.drop_scripts.iter() {
         println!("drop database: executing {:?}", path_buf);
         let prepared_script = prepare_admin_script(&script)
-            .context(format!("drop error: failed to prepare drop script {:?}", path_buf))?;
+            .context(format!("drop database error: failed to prepare drop script {:?}", path_buf))?;
         pg_admin_client.batch_execute(&prepared_script)
-            .context(format!("drop error: failed to execute drop script: {:?}", path_buf))?;
+            .context(format!("drop database error: failed to execute drop script: {:?}", path_buf))?;
     }
 
 
@@ -882,7 +895,7 @@ pub fn drop(database_project: DatabaseProject) -> anyhow::Result<()> {
         let object_type = p_object.object_type()?;
         if object_type == DatabaseObjectType::Role {
             let role_exists = exists_object(&mut pg_admin_client, p_object_id)
-                .context(format!("drop error: failed to check if role exists {:?}", p_object_id))?;
+                .context(format!("drop database error: failed to check if role exists {:?}", p_object_id))?;
             if role_exists {
                 println!("role still exists after executing all drop scripts, drop it manually or remove it from the project {:?}", p_object_id);
             }
@@ -892,3 +905,95 @@ pub fn drop(database_project: DatabaseProject) -> anyhow::Result<()> {
     return Ok(());
 }
 
+
+pub fn drop_objects(
+    database_project: &DatabaseProject,
+    drop_public_schema: bool,
+    drop_pgfine_tables: bool,
+) -> anyhow::Result<()> {
+
+    let mut pg_client = get_pg_client()
+        .context("drop objects error: failed to connect to database")?;
+
+    let db_objects = select_db_objects(&mut pg_client)?;
+    
+    let mut drop_set: HashSet<String> = HashSet::new();
+
+
+    for (db_object_id, _db_object) in db_objects.iter() {
+        drop_set.insert(db_object_id.clone());
+    }
+
+    for (p_object_id, _p_object) in database_project.objects.iter() {
+        if drop_set.contains(p_object_id) {
+            drop_set.insert(p_object_id.clone());
+        }
+    }
+
+    if !drop_public_schema {
+        drop_set.remove("schema.public");
+    }
+
+    let mut dropped: HashSet<String> = HashSet::new();
+    let mut drop_list = Vec::from_iter(drop_set.clone());
+    let mut last_error: Option<anyhow::Error> = None;
+    while drop_set.len() > 0 {
+        let dropped_len = dropped.len();
+        
+        drop_list.sort();
+        for drop_object_id in drop_list {
+            let mut visited: HashSet<String> = HashSet::new();
+            
+            let object;
+            if db_objects.contains_key(&drop_object_id) {
+                object = &db_objects[&drop_object_id];
+            } else if database_project.objects.contains_key(&drop_object_id) {
+                object = &database_project.objects[&drop_object_id];
+            } else {
+                println!("failed to drop, missing pgfine_objects {:?}", drop_object_id);
+                last_error = Some(anyhow!("failed to drop, missing pgfine_objects {:?}", drop_object_id));
+                continue;
+            }
+            
+            let drop_result = drop_object_with_deps(
+                &mut pg_client, 
+                &object, 
+                &database_project,
+                &db_objects,
+                true,
+                &mut dropped,
+                &mut visited
+            );
+
+            if drop_result.is_err() {
+                println!("failed to drop {:?}", object.id);
+                last_error = drop_result.err();
+            }
+        }
+
+        drop_set = drop_set.sub(&dropped);
+        drop_list = Vec::from_iter(drop_set.clone());
+        
+        if dropped.len() == dropped_len {
+            if let Some(e) = last_error {
+                return Err(e.context(format!("drop objects error: could not drop these objects {:?}", drop_list)));
+            } else {
+                bail!("drop objects error: could not drop these objects {:?}", drop_list);
+            }
+        }
+        if drop_set.len() > 0 {
+            println!("one more drop iteration will be attempted {:?}", drop_list);
+        }
+    }
+
+    if drop_pgfine_tables {
+        let sql_drop = "drop table pgfine_objects; drop table pgfine_migrations;";
+        pg_client.batch_execute(sql_drop)?;
+    } else {
+        let sql_truncate = "truncate table pgfine_objects; truncate table pgfine_migrations;";
+        pg_client.batch_execute(sql_truncate)?;
+    }
+
+
+    return Ok(());
+}
